@@ -577,7 +577,7 @@ static int seat_vt_event(struct uterm_vt *vt, struct uterm_vt_event *ev,
 }
 
 static void seat_input_event(struct uterm_input *input,
-			     struct uterm_input_event *ev,
+			     struct uterm_input_key_event *ev,
 			     void *data)
 {
 	struct kmscon_seat *seat = data;
@@ -766,7 +766,7 @@ int kmscon_seat_new(struct kmscon_seat **out,
 	if (ret)
 		goto err_conf;
 
-	ret = uterm_input_register_cb(seat->input, seat_input_event, seat);
+	ret = uterm_input_register_key_cb(seat->input, seat_input_event, seat);
 	if (ret)
 		goto err_input;
 
@@ -777,13 +777,18 @@ int kmscon_seat_new(struct kmscon_seat **out,
 	if (ret)
 		goto err_input_cb;
 
+	if (seat->conf->session_control && uterm_vt_get_type(seat->vt) == UTERM_VT_REAL) {
+		log_warning("session control cannot be configured on real VT, disabling session control");
+		seat->conf->session_control = false;
+	}
+
 	ev_eloop_ref(seat->eloop);
 	uterm_vt_master_ref(seat->vtm);
 	*out = seat;
 	return 0;
 
 err_input_cb:
-	uterm_input_unregister_cb(seat->input, seat_input_event, seat);
+	uterm_input_unregister_key_cb(seat->input, seat_input_event, seat);
 err_input:
 	uterm_input_unref(seat->input);
 err_conf:
@@ -829,7 +834,7 @@ void kmscon_seat_free(struct kmscon_seat *seat)
 	}
 
 	uterm_vt_deallocate(seat->vt);
-	uterm_input_unregister_cb(seat->input, seat_input_event, seat);
+	uterm_input_unregister_key_cb(seat->input, seat_input_event, seat);
 	uterm_input_unref(seat->input);
 	kmscon_conf_free(seat->conf_ctx);
 	free(seat->name);
@@ -919,12 +924,12 @@ void kmscon_seat_refresh_display(struct kmscon_seat *seat,
 	}
 }
 
-int kmscon_seat_add_input(struct kmscon_seat *seat, const char *node)
+int kmscon_seat_add_input(struct kmscon_seat *seat, const char *node, bool mouse)
 {
 	if (!seat || !node)
 		return -EINVAL;
 
-	uterm_input_add_dev(seat->input, node);
+	uterm_input_add_dev(seat->input, node, mouse);
 	return 0;
 }
 
@@ -1128,6 +1133,10 @@ int kmscon_session_set_foreground(struct kmscon_session *sess)
 
 	seat = sess->seat;
 	if (seat && seat->current_sess == sess && !seat->foreground) {
+		ret = uterm_vt_restore(seat->vt);
+		if (ret)
+			return ret;
+
 		ret = seat_go_foreground(seat, true);
 		if (ret)
 			return ret;
